import ij.IJ;
import ij.ImagePlus;
import ij.WindowManager;
import ij.gui.GenericDialog;
import ij.measure.Calibration;
import ij.plugin.PlugIn;
import ij.process.ImageProcessor;
import imagescience.images.Aspects;
import imagescience.images.ColorImage;
import imagescience.images.Dimensions;
import imagescience.images.Image;
import imagescience.transforms.Turn;
import imagescience.utilities.I5DResource;
import imagescience.utilities.Messenger;
import java.awt.image.ColorModel;

public class TJ_Turn implements PlugIn {
    
    private static final String[] angles = {"0","90","180","270"};

    private static int zindex = 0;
    private static int yindex = 0;
    private static int xindex = 0;

    public void run(String arg) {

	msg.debug(TJ_Options.debug);
	msg.debug("TransformJ "+TJ_Options.VERSION+": Turn");

	if (IJ.versionLessThan("1.35i")) return;

	final ImagePlus imp = WindowManager.getCurrentImage();
	if (imp == null)  { error("There are no images open"); return; }

	GenericDialog gd = new GenericDialog("TransformJ: Turn");
	gd.addChoice("z-angle in degrees:",angles,angles[zindex]);
	gd.addChoice("y-angle in degrees:",angles,angles[yindex]);
	gd.addChoice("x-angle in degrees:",angles,angles[xindex]);
	gd.showDialog();

	if (gd.wasCanceled()) return;

	zindex = gd.getNextChoiceIndex();
	yindex = gd.getNextChoiceIndex();
	xindex = gd.getNextChoiceIndex();

	try {
	    final Image img = Image.wrap(imp);
	    final Turn turner = new Turn();
	    turner.debug(TJ_Options.debug);
	    final Image newimg = turner.run(img,zindex,yindex,xindex);
	    ImagePlus newimp = newimg.imageplus();
	    final Aspects newasps = newimg.aspects();
	    final Calibration cal = imp.getCalibration().copy();
	    cal.pixelWidth = newasps.x;
	    cal.pixelHeight = newasps.y;
	    cal.pixelDepth = newasps.z;
	    newimp.setCalibration(cal);
	    final ImageProcessor ip = imp.getProcessor();
	    final ImageProcessor newip = newimp.getProcessor();
	    final ColorModel cm = ip.getColorModel();
	    newip.setColorModel(cm);
	    newimp.getStack().setColorModel(cm);
	    newip.setMinAndMax(ip.getMin(),ip.getMax());
	    boolean i5dinstalled = false;
	    try { Class.forName("Image5D"); i5dinstalled = true; } catch (Exception e) { }
	    if (i5dinstalled && !(newimg instanceof ColorImage)) {
		final Dimensions newdims = newimg.dimensions();
		if (newdims.t > 1 || newdims.s > 1) {
		    newimp = I5DResource.convert(newimp,true);
		    if (I5DResource.instance(imp))
			I5DResource.transfer(imp,newimp);
		}
	    }
	    newimp.changes = TJ_Options.asksave;
	    newimp.show();

	} catch (OutOfMemoryError e) {
	    error("Not enough memory for this operation");
	}
    }

    private void error(final String message) { msg.error("TransformJ: Error", message+"."); }

    private final Messenger msg = new Messenger();

}
